#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// This indicator will automatically draw fibonacci extensions from swing highs and
	/// lows in the market.
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
    [Description("Code Written by Tyler Moore - www.FreeIndicators.Com")]
    public class AutoFibExtensions : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int strength = 20; // Default setting for Strength
        // User defined variables (add any user defined variables below)
		private IFibonacciExtensions fib;
		private MIN min;
		private MAX max;
		private int barsSinceHigh = -1, barsSinceLow = -1;	
		private int pBarsSinceHigh = -1, pBarsSinceLow = -1;	
        #endregion

        protected override void Initialize()
        {
            Overlay				= true;
        }
		
		protected override void OnStartUp()
		{
			min=MIN(Low,strength);
			max=MAX(High,strength);
		}

        protected override void OnBarUpdate()
        {
            if (CurrentBar<strength)
				return;
			
			if (High[0]>max[1])
			{
				if (barsSinceLow<barsSinceHigh)
					pBarsSinceHigh = barsSinceHigh;
				barsSinceHigh=0;
			}
			else if (barsSinceHigh>-1)
				barsSinceHigh++;			
			
			if (Low[0]<min[1])
			{
				if (barsSinceHigh<barsSinceLow)
					pBarsSinceLow = barsSinceLow;
				barsSinceLow=0;
			}
			else if (barsSinceLow>-1)
				barsSinceLow++;			
			
			if (pBarsSinceLow>-1)
				pBarsSinceLow++;
			if (pBarsSinceHigh>-1)
				pBarsSinceHigh++;
			
			if (Math.Max(pBarsSinceHigh,pBarsSinceLow)<=0)
				return;
			
			if (barsSinceHigh<barsSinceLow)				
				fib=DrawFibonacciExtensions("fib",AutoScale,pBarsSinceHigh,High[pBarsSinceHigh],barsSinceLow,Low[barsSinceLow],barsSinceHigh,High[barsSinceHigh]);							
			else
				fib=DrawFibonacciExtensions("fib",AutoScale,pBarsSinceLow,Low[pBarsSinceLow],barsSinceHigh,High[barsSinceHigh],barsSinceLow,Low[barsSinceLow]);
        }

        #region Properties
        [Description("")]
        [GridCategory("Parameters")]
        public int Strength
        {
            get { return strength; }
            set { strength = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private AutoFibExtensions[] cacheAutoFibExtensions = null;

        private static AutoFibExtensions checkAutoFibExtensions = new AutoFibExtensions();

        /// <summary>
        /// Code Written by Tyler Moore - www.FreeIndicators.Com
        /// </summary>
        /// <returns></returns>
        public AutoFibExtensions AutoFibExtensions(int strength)
        {
            return AutoFibExtensions(Input, strength);
        }

        /// <summary>
        /// Code Written by Tyler Moore - www.FreeIndicators.Com
        /// </summary>
        /// <returns></returns>
        public AutoFibExtensions AutoFibExtensions(Data.IDataSeries input, int strength)
        {
            if (cacheAutoFibExtensions != null)
                for (int idx = 0; idx < cacheAutoFibExtensions.Length; idx++)
                    if (cacheAutoFibExtensions[idx].Strength == strength && cacheAutoFibExtensions[idx].EqualsInput(input))
                        return cacheAutoFibExtensions[idx];

            lock (checkAutoFibExtensions)
            {
                checkAutoFibExtensions.Strength = strength;
                strength = checkAutoFibExtensions.Strength;

                if (cacheAutoFibExtensions != null)
                    for (int idx = 0; idx < cacheAutoFibExtensions.Length; idx++)
                        if (cacheAutoFibExtensions[idx].Strength == strength && cacheAutoFibExtensions[idx].EqualsInput(input))
                            return cacheAutoFibExtensions[idx];

                AutoFibExtensions indicator = new AutoFibExtensions();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Strength = strength;
                Indicators.Add(indicator);
                indicator.SetUp();

                AutoFibExtensions[] tmp = new AutoFibExtensions[cacheAutoFibExtensions == null ? 1 : cacheAutoFibExtensions.Length + 1];
                if (cacheAutoFibExtensions != null)
                    cacheAutoFibExtensions.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheAutoFibExtensions = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Code Written by Tyler Moore - www.FreeIndicators.Com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.AutoFibExtensions AutoFibExtensions(int strength)
        {
            return _indicator.AutoFibExtensions(Input, strength);
        }

        /// <summary>
        /// Code Written by Tyler Moore - www.FreeIndicators.Com
        /// </summary>
        /// <returns></returns>
        public Indicator.AutoFibExtensions AutoFibExtensions(Data.IDataSeries input, int strength)
        {
            return _indicator.AutoFibExtensions(input, strength);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Code Written by Tyler Moore - www.FreeIndicators.Com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.AutoFibExtensions AutoFibExtensions(int strength)
        {
            return _indicator.AutoFibExtensions(Input, strength);
        }

        /// <summary>
        /// Code Written by Tyler Moore - www.FreeIndicators.Com
        /// </summary>
        /// <returns></returns>
        public Indicator.AutoFibExtensions AutoFibExtensions(Data.IDataSeries input, int strength)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.AutoFibExtensions(input, strength);
        }
    }
}
#endregion
